<?php

namespace Drupal\commerce_checkout_url\Tests;

/**
 * Test protected download.
 */
class AccessTest extends \DrupalWebTestCase {

  protected $profile = 'testing';

  /**
   * {@inheritdoc}
   */
  public static function getInfo() {
    return array(
      'name' => 'Access test',
      'description' => 'Test access to checkout page via URL.',
      'group' => 'Commerce Checkout URL',
    );
  }

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp(array(
      'commerce_cart',
      'commerce_checkout_url',
      // Product UI provides the default "product" product type.
      'commerce_product_ui',
    ));
  }

  /**
   * Creates a dummy product for use with other tests.
   */
  public function createDummyOrder($uid = 0) {
    $product = commerce_product_new('product');
    $product->sku = $this->randomName(10);
    $product->title = $this->randomName(10);
    $product->uid = 1;

    $product->commerce_price[LANGUAGE_NONE][0]['amount'] = rand(50, 500);
    $product->commerce_price[LANGUAGE_NONE][0]['currency_code'] = 'USD';

    commerce_product_save($product);

    $line_item = commerce_product_line_item_new($product, 1);
    commerce_line_item_save($line_item);

    $order = commerce_order_new($uid, 'checkout_checkout');
    $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
    $order_wrapper->commerce_line_items[] = $line_item;
    commerce_order_save($order);

    return $order;
  }

  /**
   * Tests anonymous user accessing an anonymous order.
   */
  public function testAnonymousCheckout() {
    user_role_change_permissions(DRUPAL_ANONYMOUS_RID, array(
      'access checkout' => TRUE,
    ));

    $order = $this->createDummyOrder(0);

    // Attempt to access an anonymous order directly.
    $this->drupalGet('checkout/' . $order->order_id);
    $this->assertResponse(404, 'Access to cart is denied.');

    $this->drupalGet(COMMERCE_CHECKOUT_URL_DEFAULT_MENU_ITEM_PATH . '/' . $order->order_id);
    $this->assertResponse(404, 'Access to claim form is denied without HMAC.');

    // Generate a HMAC protected URL and use that.
    $url = commerce_checkout_url_create_url($order);
    $this->assertTrue($url);
    $this->drupalGet($url);

    // Check response.
    $this->assertResponse(200, 'Access to claim form is granted with HMAC protected URL.');
    $this->assertRaw(format_string('This is a one-time link for %label. The link will expire on', array(
      '%label' => entity_label('commerce_order', $order),
    )));
    $this->assertText('Click on this button to proceed to checkout.');

    // Submit the form.
    $this->drupalPost(NULL, array(), 'Checkout');
    $this->assertResponse(200, 'Access to checkout is granted.');
    $this->assertUrl('checkout/' . $order->order_id, array(), 'Redirected to checkout form');
  }

  /**
   * Tests authenticated user accessing an anonymous order.
   */
  public function testAuthenticatedCheckout() {
    $site_user = $this->drupalCreateUser(array('access checkout'));
    $this->drupalLogin($site_user);

    $order = $this->createDummyOrder(0);

    // Attempt to access an anonymous order directly.
    $this->drupalGet('checkout/' . $order->order_id);
    $this->assertResponse(404, 'Access to cart is denied.');

    $this->drupalGet(COMMERCE_CHECKOUT_URL_DEFAULT_MENU_ITEM_PATH . '/' . $order->order_id);
    $this->assertResponse(404, 'Access to claim form is denied without HMAC.');

    // Generate a HMAC protected URL and use that.
    $url = commerce_checkout_url_create_url($order);
    $this->assertTrue($url);
    $this->drupalGet($url);

    // Check response.
    $this->assertResponse(200, 'Access to claim form is granted with HMAC protected URL.');
    $this->assertRaw(format_string('This is a one-time link for %label. The link will expire on', array(
      '%label' => entity_label('commerce_order', $order),
    )));
    $this->assertText('Click on this button to proceed to checkout.');

    // Submit the form.
    $this->drupalPost(NULL, array(), 'Checkout');
    $this->assertResponse(200, 'Access to checkout is granted.');
    $this->assertUrl('checkout/' . $order->order_id, array(), 'Redirected to checkout form');
  }

  /**
   * Tests authenticated user accessing its own order.
   */
  public function testOwnOrderCheckout() {
    $site_user = $this->drupalCreateUser(array('access checkout'));
    $this->drupalLogin($site_user);

    $order = $this->createDummyOrder($site_user->uid);

    // Attempt to access an anonymous order directly.
    $this->drupalGet('checkout/' . $order->order_id);
    $this->assertResponse(200, 'Access to own cart is granted.');

    $this->drupalGet(COMMERCE_CHECKOUT_URL_DEFAULT_MENU_ITEM_PATH . '/' . $order->order_id);
    $this->assertResponse(404, 'Access to claim form is denied without HMAC.');

    // Generate a HMAC protected URL and use that.
    $url = commerce_checkout_url_create_url($order);
    $this->assertTrue($url);
    $this->drupalGet($url);

    // Check response.
    $this->assertResponse(200, 'Access to claim form is granted with HMAC protected URL.');
    $this->assertRaw(format_string('This is a one-time link for %label. The link will expire on', array(
      '%label' => entity_label('commerce_order', $order),
    )));
    $this->assertText('Click on this button to proceed to checkout.');

    // Submit the form.
    $this->drupalPost(NULL, array(), 'Checkout');
    $this->assertResponse(200, 'Access to checkout is granted.');
    $this->assertUrl('checkout/' . $order->order_id, array(), 'Redirected to checkout form');
  }

  /**
   * Tests anonymous user attempting to access an order already owned.
   */
  public function testOwnedOrderCheckout() {
    $order_owner = $this->drupalCreateUser(array('access checkout'));
    user_role_change_permissions(DRUPAL_ANONYMOUS_RID, array(
      'access checkout' => TRUE,
    ));

    $order = $this->createDummyOrder($order_owner->uid);

    // Attempt to access an anonymous order directly.
    $this->drupalGet('checkout/' . $order->order_id);
    $this->assertResponse(404, 'Access to cart is denied.');

    $this->drupalGet(COMMERCE_CHECKOUT_URL_DEFAULT_MENU_ITEM_PATH . '/' . $order->order_id);
    $this->assertResponse(404, 'Access to claim form is denied without HMAC.');

    // Generate a HMAC protected URL and use that.
    $url = commerce_checkout_url_create_url($order);
    $this->assertTrue($url);
    $this->drupalGet($url);

    // Check response.
    $this->assertResponse(404, 'Access to claim form is denied even with HMAC protected URL.');

    // Retry with authenticated user.
    $site_user = $this->drupalCreateUser(array('access checkout'));
    $this->drupalLogin($site_user);

    // Attempt to access an owned order directly.
    $this->drupalGet('checkout/' . $order->order_id);
    $this->assertResponse(404, 'Access to cart is denied.');

    $this->drupalGet(COMMERCE_CHECKOUT_URL_DEFAULT_MENU_ITEM_PATH . '/' . $order->order_id);
    $this->assertResponse(404, 'Access to claim form is denied without HMAC.');

    // Generate a HMAC protected URL and use that.
    $url = commerce_checkout_url_create_url($order);
    $this->assertTrue($url);
    $this->drupalGet($url);

    // Check response.
    $this->assertResponse(404, 'Access to claim form is denied even with HMAC protected URL.');
  }

}
